//
#include <delays.h>
#include "Joystick.h"


// --------------------------------------------------------------------------------------
enum ADChannels
{
	CH_00 = 0b00000000,			// 0
	CH_01 = 0b00000100,			// 1 
	CH_02 = 0b00001000,			// 2 
	CH_03 = 0b00001100,			// 3
};

//
static rom byte HutTable[] =
{
	0x0F,		// 0b0000(0)	-- Zero value (HID meaning)
	0x00,		// 0b0001(1)	-- Up
	0x02,		// 0b0010(2)	-- Right
	0x01,		// 0b0011(3)	-- * Up-Right
	0x04,		// 0b0100(4)	-- Down
	0x0F,		// 0b0101(5)	-- * (Impossible)
	0x03,		// 0b0110(6)	-- * Right-Down
	0x0F,		// 0b0111(7)	-- 
	0x06,		// 0b1000(8)	-- Left
	0x07,		// 0b1001(9)	-- * Left-Up
	0x0F,		// 0b1010(10)	-- * (Impossible)
	0x0F,		// 0b1011(11)	--
	0x05,		// 0b1100(12)	-- * Down-Left
	0x0F,		// 0b1101(13)	--
	0x0F,		// 0b1110(14)	--
	0x0F,		// 0b1111(15)	--
};


// File variables
// --------------------------------------------------------------------------------------
// ADC
#define ChannelsNum		4
static WORD ADCSense[ChannelsNum];


// Hut1
 static BYTE Hut1;
// Buttons
static BYTE Buttons1;
static BYTE Buttons2;
static BYTE Buttons3;


//
// --------------------------------------------------------------------------------------
void InitIO(void)
{
	byte i = 0;

	// Axis
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	ADCON1 |= 0b00001100;           //AN0-AN3

	ADCON0=0x01;
	ADCON2=0b10111110;				// 

	// Buttons
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	// Input matrix
   	TRISBbits.TRISB0 = 0x01;
	TRISBbits.TRISB1 = 0x01;
	TRISBbits.TRISB2 = 0x01;
	TRISBbits.TRISB3 = 0x01;
	
	// Output matrix
	TRISBbits.TRISB4 = 0x00;
	TRISBbits.TRISB5 = 0x00;
	TRISBbits.TRISB6 = 0x00;
	TRISBbits.TRISB7 = 0x00; 

	PORTBbits.RB4 = 0b01;
	PORTBbits.RB5 = 0b01;
	PORTBbits.RB6 = 0b01;
	PORTBbits.RB7 = 0b01; 
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	for(; i < ChannelsNum; i++)
		ADCSense[i]._word = 0x0000;

	Hut1._byte = 0x0F;
	Buttons1._byte = 0x00;
	Buttons2._byte = 0x00;
	Buttons3._byte = 0x00;
}


//
// --------------------------------------------------------------------------------------
void SelectADChannel(byte Channel)
{
	ADCON0 &= 0b00000001;			//
	ADCON0 |= Channel;
}

//
// --------------------------------------------------------------------------------------
word ReadADC(void)
{
	WORD Result;

    ADCON0bits.GO = 1;              // Start AD conversion

//    while(ADCON0bits.NOT_DONE)
//		;							// wait for conversion continuing signal bit clear

	_asm sleep _endasm				// enter idle mode and wait for conversion end interrupt

	Result.byte0 = ADRESL;
	Result.byte1 = ADRESH;

	return Result._word;
}

//
// --------------------------------------------------------------------------------------
void WorkIOut(void)
{
	byte i = 0, j = 0, Channel;
	BYTE HutCode;
	WORD Result;

	// Axis
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	for(; i < 2; i++)
	{
		Result._word = 0;

		Channel = i << 2;
		SelectADChannel(Channel);

//		for(j = 0; j < 4; j++)
		{
			Delay10TCYx(10);
			Result._word += ReadADC();
		}

		ADCSense[i]._word = Result._word; // /4;
	}

	//
	ADCSense[0]._word -= 512;
	Report01.XL = ADCSense[0].byte0;
	Report01.XH = ADCSense[0].byte1;

	ADCSense[1]._word -= 512;
	Report01.YL = ADCSense[1].byte0;
	ADCSense[1]._word <<= 2;
	Report01.YH = ADCSense[1].byte1;

	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	Result._word = 0;
	SelectADChannel(CH_02);

//	for(j = 0; j < 4; j++)
	{
		Delay10TCYx(10);
		Result._word += ReadADC();
	}

	ADCSense[2]._word = Result._word; // /4;

//	ADCSense[2]._word -= 512;
	Report01.ZL = ADCSense[2].byte0;
	ADCSense[2]._word <<= 4;
	Report01.ZH = ADCSense[2].byte1;

	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	Result._word = 0;
	SelectADChannel(CH_03);

//	for(j = 0; j < 4; j++)
	{
		Delay10TCYx(10);
		Result._word += ReadADC();
	}

	ADCSense[3]._word = Result._word; // /4;

//	ADCSense[3]._word -= 512;
	Report01.RxL = ADCSense[3].byte0;
	ADCSense[3]._word <<= 6;
	Report01.RxH = ADCSense[3].byte1;

	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	// Hut1
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	HutCode._byte = 0x00;

	PORTBbits.RB4 = 0b00;					//row1
	Delay10TCYx(5);
	HutCode.b0 = !PORTBbits.RB0;			//Hut up
	HutCode.b1 = !PORTBbits.RB1;			//Rihgt
	HutCode.b2 = !PORTBbits.RB2;			//Down
	HutCode.b3 = !PORTBbits.RB3;			//Left
	PORTBbits.RB4 = 0b01;

	Report01.Hut1 = HutTable[HutCode._byte];

	// Buttons
	// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
	// Buttons1
	PORTBbits.RB5 = 0b00;				//row2
	Delay10TCYx(5);
	Buttons1.b0 = !PORTBbits.RB0;		//1
	Buttons1.b1 = !PORTBbits.RB1;
	Buttons1.b2 = !PORTBbits.RB2;
	Buttons1.b3 = !PORTBbits.RB3;
	PORTBbits.RB5 = 0b01;

	Report01.Buttons1 = Buttons1._byte;

	// Buttons2 
	PORTBbits.RB6 = 0b00;				//row3
	Delay10TCYx(5);
	Buttons2.b0 = !PORTBbits.RB0;
	Buttons2.b1 = !PORTBbits.RB1;
	Buttons2.b2 = !PORTBbits.RB2;
	Buttons2.b3 = !PORTBbits.RB3;
	PORTBbits.RB6 = 0b01;
	
    Report01.Buttons2 = Buttons2._byte;
    
    // Buttons3
	PORTBbits.RB7 = 0b00;				//row4
	Delay10TCYx(5);
    Buttons3.b0 = !PORTBbits.RB0;
	Buttons3.b1 = !PORTBbits.RB1;
	Buttons3.b2 = !PORTBbits.RB2;
	Buttons3.b3 = !PORTBbits.RB3;
	PORTBbits.RB7 = 0b01;

	Report01.Buttons3 = Buttons3._byte;
}
