/*********************************************************************
 *		Joystick Firmware, based heavily on
 *		Microchip USB C18 Firmware Version 1.0
 *		Rawin Rojvanit       19/11/04    Original.
 *
 *		Note: Though original code is excellent Microchip PIC18Fx550
 *		USB firmware framework, it is too complex for me to take fun
 *		with, so I reshaped it.
 *
 *		Note 2: Please, remember, this is hobby application, so do not
 *		expect industrial reliability standarts or fine polished code,
 *		though I did tried to do my best in present time margins :-)
 *		Use this as reference design only.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *	-					 	 Joystick modification, 0.01
 *	-						 	 Revision 0.01 -- USB/HID
 *										 complete.
 *	-						 	 Revision 0.01a -- 4 axis, hut
 *										 and 12 buttons complete.
 *********************************************************************/

//
#include <string.h>
#include <delays.h>

//
#include "Joystick.h"


// MCU configuration
// --------------------------------------------------------------------------------------
#pragma config LVP = OFF, MCLRE = ON, VREGEN = ON, PBADEN = OFF, WDT = OFF, IESO = OFF
#pragma config PLLDIV = 5, CPUDIV = OSC1_PLL2, USBDIV = 2, FOSC = HSPLL_HS


// Descriptors
// --------------------------------------------------------------------------------------
#pragma romdata

// Device Descriptor
rom DeviceDescriptorFormat DeviceDescriptor =
{    
    sizeof(DeviceDescriptorFormat),		// Size of this descriptor in bytes
    DEVICE,								// DEVICE descriptor type
	0x0200,								// 2.0 USB Spec Release Number in BCD format
    0x00,								// Class Code
    0x00,								// Subclass code
    0x00,								// Protocol code
    EP0_BufferSize,						// Max packet size for EP0, see usbcfg.h
   	0x04D8,								// Vendor ID: 0x0000 reserved for development
    0x0001,								// Product ID:
    0x0008,								// Device release number in BCD format
    0x01,								// Manufacturer string index
    0x02,								// Product string index
    0x00,								// Device serial number string index
    0x01								// Number of possible configurations
};

// Report descriptors
// --------------------------------------------------------------------------------------
// Joystick 01
#define REPORT_SIZE_01			111
rom byte ReportDescriptor_01[REPORT_SIZE_01] =
{
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,                    // USAGE (Joystick)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x16, 0x00, 0xfe,              //     LOGICAL_MINIMUM (-512)
    0x26, 0xff, 0x01,              //     LOGICAL_MAXIMUM (511)
    0x75, 0x0a,                    //     REPORT_SIZE (10)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x31,                    //     USAGE (Y)
    0x16, 0x00, 0xfe,              //     LOGICAL_MINIMUM (-512)
    0x26, 0xff, 0x01,              //     LOGICAL_MAXIMUM (511)
    0x75, 0x0a,                    //     REPORT_SIZE (10)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0xc0,                          //     END_COLLECTION
    0x05, 0x01,                    //   USAGE_PAGE (Generic Desktop)
    0x09, 0x32,                    //   USAGE (Z)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x03,              //   LOGICAL_MAXIMUM (1023)
    0x75, 0x0a,                    //   REPORT_SIZE (10)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x05, 0x01,                    //   USAGE_PAGE (Generic Desktop)
    0x09, 0x33,                    //   USAGE (Rx)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x03,              //   LOGICAL_MAXIMUM (1023)
    0x75, 0x0a,                    //   REPORT_SIZE (10)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x05, 0x01,                    //   USAGE_PAGE (Generic Desktop)
    0x09, 0x39,                    //   USAGE (Hat switch)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x07,                    //   LOGICAL_MAXIMUM (7)
    0x35, 0x00,                    //   PHYSICAL_MINIMUM (0)
    0x46, 0x3b, 0x01,              //   PHYSICAL_MAXIMUM (315)
    0x65, 0x14,                    //   UNIT (Eng Rot:Angular Pos)
    0x75, 0x04,                    //   REPORT_SIZE (4)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x0c,                    //   USAGE_MAXIMUM (Button 12)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x95, 0x0c,                    //   REPORT_COUNT (12)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

// HID Descriptor
// --------------------------------------------------------------------------------------
rom HIDFormat1 HIDDescriptor1 =
{
	sizeof(HIDFormat1),
	CLASS_HID,
	0x0101,										// HID Spec Release Number in BCD format
	0x00,										// Country code (0x00 for "not supported")
	0x02,										// Description number to request
	CLASS_REPORT,								// Descriptor type to request
	REPORT_SIZE_01,								// Size to request, word
	CLASS_HID,									// Descriptor type to request
	sizeof(HIDFormat1),							// Size to request, word
};

// Configuration 1 Descriptor
// --------------------------------------------------------------------------------------
rom USBConfigurationStruct ConfigurationDescriptor =
{
    // Configuration Descriptor
    sizeof(DeviceConfigurationFormat),    		// Size of this descriptor in bytes
    CONFIGURATION,                				// CONFIGURATION descriptor type
    sizeof(USBConfigurationStruct),				// Total length of data for this cfg
												// sizeof(DeviceConfigurationFormat) + sizeof(DeviceInterfaceFormat) + ...
    0x01,                      					// Number of interfaces in this cfg
    0x01,                      					// Index value of this configuration
    0x05,                      					// Configuration string index
    _DEFAULT,		          					// Attributes, see usbdefs_std_dsc.h
    50,                     					// Max power consumption (2X mA)

	// Interface descriptor
	sizeof(DeviceInterfaceFormat),				// Size of this descriptor in bytes
	INTERFACE,									// INTERFACE descriptor type
	0x00,										// Interface number (zero based)
	0x00,										// Alternate setting
	0x01,										// Number of endpoints
	0x03,										// Interface class (0x3 is HID
	0x00,										// Interface subclass
	0x00,										// Protocol
	0x04,										// Interface Sting index

	// HID -- Just have to repeat initialisation here, cose scilless in "C" :-)
	sizeof(HIDFormat),
	CLASS_HID,
	0x0110,										// HID Spec Release Number in BCD format
	  32,										// Country code (0x00 for "not supported")
	0x01,										// Number of descriptors to request
	CLASS_REPORT,								// Descriptor type to request
	REPORT_SIZE_01,								// Size to request, word
//	CLASS_HID,									// Descriptor type to request
//	sizeof(HIDFormat1),							// Size to request, word

	// Endpoint
	sizeof(EndPointFormat),				// Size of this descriptor in bytes
	ENDPOINT,									// ENDPOINT descriptor type
	0b10000001,									// Endpoint address
	0b00000011,									// Attributes
	16,											// Max packet size
	0x01										// Polling interval, ms
};


// Strings (WinXP SP2 will never ask all of this, except strings #0 and #2, but these it will ask 4 times 8-)
// And so, we need to use custom .inf file to see our nice and eyecatching strings elsewere (see setupapi.log).
// --------------------------------------------------------------------------------------
rom struct{byte bLength; byte bDscType; word string[2];} LangIDs = { sizeof(LangIDs), STRING, 0x0409, 0x419 };		// 0

rom struct{byte bLength; byte bDscType; word string[25];} MicrochipStr = { sizeof(MicrochipStr), STRING,						// 1
'M','i','c','r','o','c','h','i','p',' ','T','e','c','h','n','o','l','o','g','y',' ','I','n','c','.' };
rom struct{byte bLength; byte bDscType; word string[13];} LocationStr = { sizeof(LocationStr), STRING,				// 2-
'J','o','y','s','t','i','c','k',' ','v','1','.','0'};
rom struct{byte bLength; byte bDscType; word string[19];} ProductStr = { sizeof(ProductStr), STRING,				// 2
'J','o','y','s','t','i','c','k',',',' ','r','e','v','.',' ','4','.','12','1' };
rom struct{byte bLength; byte bDscType; word string[4];} SerialStr = { sizeof(SerialStr), STRING,					// 3
'0','0','0','1' };
rom struct{byte bLength; byte bDscType; word string[12];} InterfaceStr = { sizeof(InterfaceStr), STRING,			// 4
'J','o','y','s','t','i','c','k',' ','-',' ','1' };
rom struct{byte bLength; byte bDscType; word string[12];} ConfigStr = { sizeof(ConfigStr), STRING,					// 5
'J','o','y','s','t','i','c','k',' ','-',' ','1a' };


//	U S B  F I X E D  L O C A T I O N  V A R I A B L E S
// --------------------------------------------------------------------------------------
#pragma udata usbram4=0x400     			//See Linker Script,usb4:0x400-0x4FF(256-byte)

volatile far BDTable EP0_Out;				//Endpoint #0 BD Out
volatile far BDTable EP0_In;				//Endpoint #0 BD In
volatile far BDTable EP1_Out;
volatile far BDTable EP1_In;


volatile far SetupDataFormat SetupData;
volatile far byte EP0_Data[8];

volatile far byte EP1_OutData[EP1_BufferSize];
volatile far byte EP1_InData[EP1_BufferSize];


#pragma udata
// Global variables
// --------------------------------------------------------------------------------------
volatile far ReportFormat01 Report01;

// Log
byte Stopped = 0;
byte LogIndex = 0;
byte Log[LOGSIZE];


// File variables
// --------------------------------------------------------------------------------------
static byte USBDeviceState = DETACHED_STATE;				// Device States: DETACHED, ATTACHED, ...
static byte Need2Stop = 0;


// Interrupt handlers
// --------------------------------------------------------------------------------------
#pragma interrupt high_isr
void high_isr(void)
{
	if(PIR1bits.ADIF)
		PIR1bits.ADIF = 0b00;

	INTCONbits.GIEH = 0b01;		// Enables further high priority interrupts
}

//
#pragma interruptlow low_isr
void low_isr(void)
{
//	if(PIR1bits.ADIF)
//		PIR1bits.ADIF = 0b00;

	INTCONbits.GIEL = 0b01;		// Enables further low priority interrupts
}

// Interrupt vectors
// --------------------------------------------------------------------------------------
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
    _asm goto high_isr _endasm
}
#pragma code

//
#pragma code low_vector=0x18
void interrupt_at_low_vector(void)
{
    _asm goto low_isr _endasm
}

// 
#pragma code

//
// --------------------------------------------------------------------------------------
void Init(void)
{
	// Setup idle mode
	OSCCONbits.IDLEN =  0b01;

	// Setup global interrupts
	RCONbits.IPEN = 0b01;

	INTCONbits.GIEH = 0b01;
	INTCONbits.GIEL = 0b01;

	// Setup peripheral interrupts
	PIE1bits.ADIE = 0b01;
}


#define TransferBlinkDelay		300
//
// --------------------------------------------------------------------------------------
void ServeEP1(USBStatusReg USBStatus)
{
	static unsigned short int GreenCount = TransferBlinkDelay;
	static int j = 0;
	byte i = 0;

//	RedLed(Off);
//	YellowLed(Off);
//	GreenLed(Free);
//
//	if(GreenCount++ >= TransferBlinkDelay)
//	{
//		GreenCount = 0;
//		PORTBbits.RB2 = ~LATBbits.LATB2;
//	}

	memcpy(EP1_InData, Report01._byte, EP1_BufferSize);

	EP1_In.Address = EP1_InData;
	EP1_In.Count = EP1_BufferSize;

	if(EP1_In.Status.DTS == 0)				// Toggle DTS bit and turn ownership to SIE
		EP1_In.Status._byte = _USIE | _DAT1 | _DTSEN;
	else
		EP1_In.Status._byte = _USIE | _DAT0 | _DTSEN;

//	Stopped = 1;		// For streaming log 2 eeprom after full setup
}

//
// --------------------------------------------------------------------------------------
void ServeEP0(USBStatusReg USBStatus)
{
	static rom byte *DataPointer = 0;
	static byte DataCount = 0;
	static byte USBAddress = 0;

	if(USBStatus.Dir == OUT)
	{
		switch(EP0_Out.Status.PID)
		{
			case SETUP_TOKEN:
//				if(USBDeviceState >= ADDRESSED_STATE)
//				{
//					byte2log(0xAA);
//					ram2log((byte *)&SetupData, EP0_BufferSize);
//				}
//
				switch(SetupData.bmRequestType)
				{
					case bmGET_xxxx:
//						RedLed(Off);		// 0x0010
//						YellowLed(On);
//						GreenLed(Off);
//
						switch(SetupData.bRequest)
						{
							case bGET_DESCRIPTOR:
								switch(SetupData.DescriptorType)
								{
									case DEVICE:
										memcpypgm2ram(EP0_Data, (rom byte *)&DeviceDescriptor, EP0_BufferSize);
										DataPointer = (rom byte *)&DeviceDescriptor;
										DataCount = sizeof(DeviceDescriptorFormat);
									break;

									case CONFIGURATION:
										memcpypgm2ram(EP0_Data, (rom byte *)&ConfigurationDescriptor, EP0_BufferSize);
										DataPointer = (rom byte *)&ConfigurationDescriptor;
										if(SetupData.DataSize >= sizeof(USBConfigurationStruct))
											DataCount = sizeof(USBConfigurationStruct);
										else
											DataCount = SetupData.DataSize;
									break;

//									case SOME01:
//										memcpypgm2ram(EP0_Data, (rom byte *)&ConfigurationDescriptor, EP0_BufferSize);
//										DataPointer = (rom byte *)&ConfigurationDescriptor;
//										DataCount = 0x0A;
//									break;
//
									case STRING:
										switch(SetupData.bIndex)
										{
											case 0:		// LangIDs
												memcpypgm2ram(EP0_Data, (rom byte *)&LangIDs, sizeof(LangIDs));
												DataPointer = (rom byte *)&LangIDs;
												DataCount = sizeof(LangIDs);
											break;

											case 1:		// Name
												memcpypgm2ram(EP0_Data, (rom byte *)&MicrochipStr, sizeof(MicrochipStr));
												DataPointer = (rom byte *)&MicrochipStr;
												DataCount = sizeof(MicrochipStr);
											break;

//											case 2:		// Product name
//												memcpypgm2ram(EP0_Data, (rom byte *)&ProductStr, sizeof(ProductStr));
//												DataPointer = (rom byte *)&ProductStr;
//												DataCount = sizeof(ProductStr);
//											break;

											case 2:		// 2- Location string
												memcpypgm2ram(EP0_Data, (rom byte *)&LocationStr, sizeof(LocationStr));
												DataPointer = (rom byte *)&LocationStr;
												DataCount = sizeof(LocationStr);
											break;

											case 3:		// Serial number
												memcpypgm2ram(EP0_Data, (rom byte *)&SerialStr, sizeof(SerialStr));
												DataPointer = (rom byte *)&SerialStr;
												DataCount = sizeof(SerialStr);
											break;

											case 4:		// Interface name
												memcpypgm2ram(EP0_Data, (rom byte *)&InterfaceStr, sizeof(InterfaceStr));
												DataPointer = (rom byte *)&InterfaceStr;
												DataCount = sizeof(InterfaceStr);
											break;

											case 5:		// Configuration name
												memcpypgm2ram(EP0_Data, (rom byte *)&ConfigStr, sizeof(ConfigStr));
												DataPointer = (rom byte *)&ConfigStr;
												DataCount = sizeof(ConfigStr);
											break;

											default:	// Default string
												memcpypgm2ram(EP0_Data, (rom byte *)&ProductStr, sizeof(ProductStr));
												DataPointer = (rom byte *)&ProductStr;
												DataCount = sizeof(ProductStr);
//												Need2Stop = 1;
										}
									break;

									default:
										EP0_In.Status._byte = _USIE | _BSTALL;
										EP0_Out.Status._byte = _USIE | _BSTALL;
										EP0_In.Count = 0;

										UCONbits.PKTDIS = 0;
										return;
								}

							    EP0_In.Address = EP0_Data;
								if(DataCount < EP0_BufferSize)
									EP0_In.Count = DataCount;
								else
								    EP0_In.Count = EP0_BufferSize;
							    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
							break;

							default:
								;
						}
					break;

					case bmSET_xxxx:
						switch(SetupData.bRequest)
						{
							case bSET_ADDRESS:							// ADDRESSED
//								RedLed(Off);		// 0x0011
//								YellowLed(Off);
//								GreenLed(On);
		
								USBAddress = SetupData.USBAddress;
								USBDeviceState = ADR_PENDING_STATE;
							break;

							case bSET_CONFIGURATION:					// CONFIGURED
						    	UEP1bits.EPHSHK = 1;
							   	UEP1bits.EPCONDIS = 1;
						    	UEP1bits.EPOUTEN = 1;
						    	UEP1bits.EPINEN = 1;

								EP1_In.Address = EP1_InData;
							    EP1_In.Count = EP1_BufferSize;
								EP1_In.Status._byte = _UCPU;

								USBDeviceState = CONFIGURED_STATE;
							break;

							default:
								;
						}

						DataCount = 0;
					    EP0_In.Address = EP0_Data;			
					    EP0_In.Count = 0;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
					break;

					// Class descriptor
					case bmGET_INTERFACE:
						switch(SetupData.bRequest)
						{
							case bGET_DESCRIPTOR:
								switch(SetupData.DescriptorType)
								{
									case CLASS_HID:
										memcpypgm2ram(EP0_Data, (byte *)&HIDDescriptor1, EP0_BufferSize);
										DataPointer = (rom byte *)&HIDDescriptor1;
										DataCount = sizeof(HIDFormat1);
									break;

									case CLASS_REPORT:
										memcpypgm2ram(EP0_Data, ReportDescriptor_01, EP0_BufferSize);
										DataPointer = ReportDescriptor_01;
										DataCount = REPORT_SIZE_01;
									break;

									default:
										;
								}
						}
					    EP0_In.Address = EP0_Data;
						if(DataCount < EP0_BufferSize)
							EP0_In.Count = DataCount;
						else
						    EP0_In.Count = EP0_BufferSize;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;

					break;

					// Class (HID)
					case CLASS_HID:
						switch(SetupData.bRequest)
						{
							case bHID_SET_IDLE:		// Just start EP1 pipe here, not important to serve
								if(SetupData.wValue == 0)
								EP1_In.Status._byte = _USIE | _DAT0; // _UCPU | _DAT1;
							break;

							default:
								;
						}

						DataCount = 0;
					    EP0_In.Address = EP0_Data;
					    EP0_In.Count = 0;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
					break;

					default:
						;
				}
				UCONbits.PKTDIS = 0;			//Enable transfer
			break;

			case OUT_TOKEN:
//				RedLed(Off);		// 0x0100
//				YellowLed(On);
//				GreenLed(On);
//
//				if(USBDeviceState >= ADDRESSED_STATE)
//				{
//					byte2log(0xCC);
//				}

			    EP0_Out.Count = EP0_BufferSize;
			    EP0_Out.Address = (byte*)&SetupData;
			    EP0_Out.Status._byte = _USIE | _DAT1 | _DTSEN;
			break;

			default:
				;
		}
	}
	else if(USBStatus.Dir == IN)
	{
		switch(EP0_In.Status.PID)
		{
			case IN_TOKEN:
//				RedLed(On);		// 0x0101
//				YellowLed(Off);
//				GreenLed(On);
//
//				if(USBDeviceState >= ADDRESSED_STATE)
//				{
//					byte2log(0xBB);
//					if(DataCount > EP0_BufferSize)
//						ram2log(EP0_Data, EP0_BufferSize);
//					else
//						ram2log(EP0_Data, DataCount);
//				}
//
				if(USBDeviceState == ADR_PENDING_STATE)
				{
					USBDeviceState = ADDRESSED_STATE;
					UADDR = USBAddress;

//					byte2log(0xAC);
//					byte2log(UADDR);
				}

				EP0_Out.Address = (byte*)&SetupData;
				EP0_Out.Count = EP0_BufferSize;
				EP0_Out.Status._byte = _USIE;

				EP0_In.Address = EP0_Data;

				DataPointer += EP0_BufferSize;
				DataCount = DataCount - EP0_BufferSize;

				if(DataCount > EP0_BufferSize)
				{
					memcpypgm2ram(EP0_Data, DataPointer, EP0_BufferSize);
					EP0_In.Count = EP0_BufferSize;
				}
				else
				{
					memcpypgm2ram(EP0_Data, DataPointer, DataCount);
					EP0_In.Count = DataCount;
				}

			    if(EP0_In.Status.DTS == 0)
					EP0_In.Status._byte = _USIE| _DAT1 | _DTSEN;
		        else
			        EP0_In.Status._byte = _USIE | _DAT0 | _DTSEN;

//				if(Need2Stop)
//					Stopped = 1;
			break;

			default:
				;
		}
	}
}


//
// --------------------------------------------------------------------------------------
void PollAndServeUSB(void)
{
	USBStatusReg USBStatus;
//	static byte Logged = 0;

//	if(Stopped && Logged)
//		return;
//
//	if(Stopped)
//	{
//		RedLed(On);		// 0x0111
//		YellowLed(On);
//		GreenLed(On);
//
//		byte2log(0xFD);
//
//		stream_ram2eeprom(Log, LogIndex);
//		Logged = 1;
//
//		return;
//	}

	//
	if(UCONbits.USBEN == 0)
	{
	    UCON = 0;
    	UIE = 0;                                // Mask all USB interrupts
	    UCONbits.USBEN = 1;                     // Enable module & attach to bus
	    USBDeviceState = ATTACHED_STATE;
	}

	//
    if(USBDeviceState == ATTACHED_STATE)
    {
		if(!UCONbits.SE0)
		{
			UIR = 0;                        	// Clear all USB interrupts
			UIE = 0;                        	// Mask all USB interrupts
	        UIEbits.URSTIE = 1;             	// Unmask RESET interrupt
	        UIEbits.IDLEIE = 1;					// Unmask IDLE interrupt
	        USBDeviceState = POWERED_STATE;
	     }										// else wait until SE0 is cleared
	}

	//
	if(UIRbits.URSTIF && UIEbits.URSTIE)		// Reset
	{
//		RedLed(Off);		// 0x0000
//		YellowLed(Off);
//		GreenLed(Off);
//
//		byte2log(0xFA);							// Decided not to log anything till addressed

    	UEIR = 0;                       		// Clear all USB error flags
    	UIR = 0;                        		// Clears all USB interrupts
    	UEIE = 0b10011111;              		// Unmask all USB error interrupts
    	UIE = 0b01111011;               		// Enable all interrupts except ACTVIE
    
    	UADDR = 0x00;                   		// Reset to default address
       
		UEP1=0x00;UEP2=0x00;UEP3=0x00;			// Reset all non-EP0 UEPn registers
    	UEP4=0x00;UEP5=0x00;UEP6=0x00;UEP7=0x00;
    	UEP8=0x00;UEP9=0x00;UEP10=0x00;UEP11=0x00;
    	UEP12=0x00;UEP13=0x00;UEP14=0x00;UEP15=0x00;

		UEP0 = 0x00;							// Init EP0 as a Ctrl EP
    	UEP0bits.EPHSHK = 1;
    	UEP0bits.EPCONDIS = 0;
    	UEP0bits.EPOUTEN = 1;
    	UEP0bits.EPINEN = 1;
 
    	while(UIRbits.TRNIF == 1)				// Flush any pending transactions
        	UIRbits.TRNIF = 0;

    	UCONbits.PKTDIS = 0;					// Make sure packet processing is enabled

	    EP0_Out.Count = EP0_BufferSize;
	    EP0_Out.Address = (byte*)&SetupData;
	    EP0_Out.Status._byte = _USIE | _DAT0 | _DTSEN;
	    EP0_In.Status._byte = _UCPU;

	    USBDeviceState = DEFAULT_STATE;
	}

	if(UIRbits.IDLEIF && UIEbits.IDLEIE)	// Idle
	{
		UIRbits.IDLEIF = 0;
	}

	if(UIRbits.SOFIF && UIEbits.SOFIE)		// SOF
	{
	    UIRbits.SOFIF = 0;
	}

    if(UIRbits.STALLIF && UIEbits.STALLIE)
	{
		UIRbits.STALLIF = 0;

//		YellowLed(Blink);
//		byte2log(0xFC);

//		Stopped = 1;
	}

	if(UIRbits.UERRIF && UIEbits.UERRIE)	// Error
	{
//		RedLed(Blink);

//		byte2log(0xFB);
//		byte2log(UEIR);

//		Stopped = 1;
	}

    if(UIRbits.TRNIF && UIEbits.TRNIE)		// Transfer
    {
		USBStatus._byte = USTAT;

		switch(USBStatus.Endpoint)
		{
			case 0:
				ServeEP0(USBStatus);
			break;

			case 1:
				ServeEP1(USBStatus);
			break;

			default:
				;
		}
		UIRbits.TRNIF = 0;
	}
}


//
// --------------------------------------------------------------------------------------
void main(void)
{
	Init();
//	InitLeds();
	InitIO();
	InitUSB();

	while(1)
	{
//		DriveLeds();
		WorkIOut();
		PollAndServeUSB();			// SIE actually starts just here
	}
}


//	if(EP1_In.Status.UOWN)					// SIE owns memory
//		return;
